SuperStrict

' slim down app
Framework BRL.GLMax2D
Import BRL.Retro
Import BRL.Timer
Import BRL.LinkedList
Import BRL.Random

' game constants
Const WIDTH%				= 800
Const HEIGHT%				= 600
Const FRAMERATE%			= 50
Const NEXTRA_INTERVAL%		= 5000
Const NEXTRA_ANOUNCE_TIME%	= 2000
Const NEXTRA_INFO_TIME%		= 2000
Global CTRLNAMES$[]			= ["D","F","J","K"]
Global CTRLS%[]				= [KEY_D, KEY_F, KEY_J, KEY_K]
Global COLS%[][]			= [[255,0,0],[0,255,0],[255,255,0],[0,128,255]]
Const NGRAVITY!				= 0.02
Const XGRAVITY!				= 0.03
Const NAVEL!				= 1
Const XAVEL!				= 1.5
' states
Const STATE_INIT%		= 0
Const STATE_WAIT%		= 1
Const STATE_PLAY%		= 2
Const STATE_DEATH%		= 3

' player values
Global xpos!, ypos!
Global xvel!, yvel!, xacc!, yacc!
Global ang!
Global avel!
Global order%[]
Global best%
' environment values
Global gravity!
' game control
Global state% = STATE_INIT
Global nextrastr$, nextratm%, nextradisptm%
Global tmstart%, tmplay%
Global timer:TTimer = CreateTimer(FRAMERATE)

'-------------------------------------------------
' app
AppTitle = "DFJK Nextra"
SeedRnd(MilliSecs())
Graphics(WIDTH, HEIGHT)
' main
Global runs%=1
While Not (KeyDown(KEY_ESCAPE) Or AppTerminate())
	Cls
	' state machine
	Select state
		Case STATE_INIT
			DoInit()
		Case STATE_WAIT
			DoWait()
		Case STATE_PLAY
			DoPlay()
		Case STATE_DEATH
			DoDeath()
	End Select
	' draw stuff
	TFlame.DrawAll()
	DrawFig()
	DrawHud()
	' synchronise
	runs = WaitTimer(timer)
	Flip 0
Wend
End

'-------------------------------------------------
Rem
	Initialise all values for new game
End Rem
Function DoInit()
	xpos = WIDTH /2
	ypos = HEIGHT /2
	xvel = 0
	yvel = 0
	ang = 0
	avel = 0
	gravity = NGRAVITY
	order = [0,1,2,3]
	state = STATE_WAIT
End Function

Rem
	Wait for user input to begin game
End Rem
Function DoWait()
	' check inputs (keys)
	For Local i% = 0 To 3
		If KeyHit(CTRLS[i]) then
			state = STATE_PLAY
			tmstart = MilliSecs()
			nextratm = tmstart +NEXTRA_INTERVAL
			nextradisptm = 0
			Exit
		EndIf
	Next
	' draw info to hit key
	SetColor(255,255,255)
	DrawTextMid("hit D,F,J or K to start", 400,250)
End Function

Rem
	Update game
	Control player / figure
	Control nextras
End Rem
Function DoPlay()
	' update playtime
	tmplay = MilliSecs() -tmstart
	' control player (sync)
	For Local i% = 0 Until runs
		UpdateFig()
	Next
	' spawn nextras if nescessary
	If MilliSecs() >= nextratm then
		RandomNextra()
		nextratm :+ NEXTRA_INTERVAL
	EndIf
End Function

Rem
	Be dead
	Wait for all keys to be released, then allow new game
End Rem
Function DoDeath()
	' zero all keyhits
	FlushKeys()
	' try to switch to new game
	state = STATE_INIT
	' keep back if any control is still pressed
	For Local i% = 0 To 3
		If KeyDown(CTRLS[i]) then
			state = STATE_DEATH
		EndIf
	Next
End Function

Rem
	Control figure
	Kill, if out of boundaries
End Rem
Function UpdateFig()
	' reset / set acceleration vector
	xacc = 0
	yacc = gravity
	' control figure
	Local a! = ang
	For Local i% = 0 To 3
		' accelerate if according key is pressed
		' Every angle is mapped to a key 0-3 (order[i]),
		' the key code is mapped in CTRLS[key].
		If KeyDown(CTRLS[order[i]]) then
			xacc :- cos(a) *0.1
			yacc :- sin(a) *0.1
			' spawn some flames (gfx)
			New TFlame.Init(xpos+cos(a)*15, ypos+sin(a)*15, cos(a)*2+Rnd(-0.2,0.2), sin(a)*2+Rnd(-0.2,0.2))
		EndIf
		a :+ 90
	Next
	' move figure
	ang :+ avel
	xvel :+ xacc
	yvel :+ yacc
	xpos :+ xvel
	ypos :+ yvel
	' check for boundaries
	If xpos < 0 Or xpos >= WIDTH Or ypos < 0 Or ypos >= HEIGHT
		' ... out of screen
		' check for new highscore
		If tmplay > best then
			best = tmplay
		EndIf
		' die
		state = STATE_DEATH
	EndIf
End Function

Rem
	Draw figure
End Rem
Function DrawFig()
	SetColor(255,255,255)
	DrawOval xpos-15, ypos-15, 31, 31
	' draw nozzles
	Local a! = ang
	For Local i% = 0 To 3
		' Every angle is mapped to a key 0-3 (order[i]),
		' the colour is mapped in COLS[key], the
		' colour values are stored in COLS second dimension.
		SetColor(COLS[order[i]][0], COLS[order[i]][1], COLS[order[i]][2])
		DrawOval xpos+cos(a)*15-3, ypos+sin(a)*15-3, 7,7
		a :+ 90
	Next
End Function

Rem
	Draw Hud
	- key indicators
	- playtime
	- highscore
	- nextra info texts
	- nextra announcements
End Rem
Function DrawHud()
	' key indicators
	Local x%=180, y%=560
	For Local i% = 0 To 3
		SetColor(COLS[i][0], COLS[i][1], COLS[i][2])
		DrawOval x-20, y-20, 41,41
		SetColor(0,0,0)
		DrawTextMid(CTRLNAMES[i], x,y)
		' move x position
		' leap after key 1 (between 0,1 and 2,3)
		x :+ 100 +140*(i=1)
	Next
	' draw highscore (best time)
	If best>0 then
		SetColor(128,128,128)
		DrawTextMid("your best: "+FTime(best), 400,40)
	EndIf
	' these infos occur only if playing
	If state = STATE_PLAY then
		' playtime
		SetColor(128,128,128)
		DrawTextMid(FTime(tmplay), 400,560)
		' announce nextra
		Local ndiff% = nextratm -MilliSecs()
		If ndiff < NEXTRA_ANOUNCE_TIME then
			DrawTextMid("NEXTRA in "+FTime(ndiff, False), 400,275)
		EndIf
		' nextra info text
		If MilliSecs() <= nextradisptm then
			DrawTextMid(nextrastr, 400,275)
		EndIf
	EndIf
End Function

Rem
	Draw a text horizontally and vertically centered around pX and pY
End Rem
Function DrawTextMid(pText$, pX%,pY%)
	SetScale(2,2)
	DrawText(pText, pX-TextWidth(pText),pY-12)
	SetScale(1,1)
End Function

Rem
	Format a time (in ms) into string of format m:ss.t
	or ss.t if pLong is false
End Rem
Function FTime$(pTime%, pLong%=True)
	Local m%,s%,t%
	' tenth
	t = (pTime/100) Mod 10
	' seconds
	s = (pTime/1000) Mod 60
	' minutes
	m = pTime/60000
	' format time
	If pLong then
		Return m+":"+Right("0"+s,2)+"."+t
	Else
		Return s+"."+t
	EndIf
End Function

Rem
	Activate random nextra
	only new (not already applied effects) will be chosen
End Rem
' foundnextra is set when NameNextra is called, thus global
Global foundnextra%
Function RandomNextra()
	' reset foundnextra, because there's not one found yet
	foundnextra = False
	' try finding a new nextra effect
	Repeat
		Local n% = Rand(0,11)
		' turn rotation off
		If n=0 And avel<>0 then
			avel = 0
			NameNextra("ROTATION OFF")
		' clockwise rotation
		ElseIf n=1 And avel<=0 then
			avel = NAVEL
			NameNextra("CLOCKWISE")
		' counterclockwise rotation
		ElseIf n=2 And avel>=0 then
			avel = -NAVEL
			NameNextra("COUNTERCLOCKWISE")
		' fast clockwise rotation
		ElseIf n=3 And avel<=0 then
			avel = XAVEL
			NameNextra("FAST CLOCKWISE")
		' fast counterclockwise rotation
		ElseIf n=4 And avel>=0 then
			avel = -XAVEL
			NameNextra("FAST COUNTERCLOCKWISE")
		' turn gravity off
		ElseIf n=6 And gravity<>0 then
			gravity = 0
			NameNextra("GRAVITY OFF")
		' normal gravity
		ElseIf n=7 And gravity<=0 then
			gravity = NGRAVITY
			NameNextra("NORMAL GRAVITY")
		' inversed gravity
		ElseIf n=8 And gravity>=0 then
			gravity = -NGRAVITY
			NameNextra("INVERSE GRAVITY")
		' strong gravity
		ElseIf n=9 And gravity<=0 then
			gravity = XGRAVITY
			NameNextra("INSANE GRAVITY")
		' stronge inversed gravity
		ElseIf n=10 And gravity>=0 then
			gravity = -XGRAVITY
			NameNextra("SUPER NEGATIVE GRAVITY")
		' switch two keys
		ElseIf n=11
			' first key (can be any one)
			Local i1% = Rand(0,3)
			' second key (must be different from first)
			Local i2% = (i1+Rand(1,3)) Mod 4
			' swap values of keys
			Local v1% = order[i1], v2% = order[i2]
			order[i1] = v2
			order[i2] = v1
			NameNextra("SWITCHED "+CTRLNAMES[i1]+" AND "+CTRLNAMES[i2])
		EndIf
	' quit if one nextra was activated, otherwise, try again
	Until foundnextra = True
End Function

Rem
	Activate nextra display
	Set foundnextra to true
End Rem
Function NameNextra(pName$)
	nextrastr = pName
	nextradisptm = MilliSecs() +NEXTRA_INFO_TIME
	foundnextra = True
End Function

'-------------------------------------------------
Rem
	Type for flames (gfx)
End Rem
Type TFlame
	Global flames:TList = CreateList()
	' flame vectors and life
	Field x!,y!, vx!,vy!
	Field life%
	Method Init:TFlame(pX!,pY!,pVX!,pVY!)
		life = 128
		x = pX
		y = pY
		vx = pVX
		vy = pVY
		flames.AddLast(Self)
		Return Self
	End Method
	
	' draw all flames
	Function DrawAll()
		For Local f:TFlame = EachIn flames
			f.Draw()
		Next
	End Function
	
	' draw flame, remove when passed
	Method Draw()
		' move
		x :+ vx
		y :+ vy
		' draw, colour gradient according to life
		SetColor(life*2,life,0)
		DrawOval x-1,y-1, 3,3
		' decrease life
		life :- 1
		' remove if passed
		If life < 0 then
			flames.Remove(Self)
		EndIf
	End Method
End Type
